
// FILE: binSearch.cpp

#include <iostream.h>
#include <exception.h>
#include "binSearch.h"
#include "printArray.h"

static void doTest(keyIndex (*search) (int, int[], int, int), int key,int num[], int low, int high);

void testSearch()
{
	int a0[] = { 3, 5, 7, 8, 13, 21, 33 };
	doTest(binSearch,7, a0, 0, 6);	
	doTest(binSearch,20, a0, 0, 6);	
	doTest(binSearch,33, a0, 0, 6);	

	doTest(ibinSearch,7, a0, 0, 6);	
	doTest(ibinSearch,20, a0, 0, 6);	
	doTest(ibinSearch,33, a0, 0, 6);	
}

void doTest(keyIndex (*search) (int, int[], int, int), int key,int num[], int low, int high)
{
	try{
		keyIndex result = search(key,num,low,high);
		cout << "binSearch("
			<< key << ", ";
		printArray(cout,num,(1+high-low)); 
		cout << ") = ";
		if (result.keyFound) {
			cout << result.index << endl;
		} else {
			cout << "not found" << endl;
		}
	} catch (xmsg &err) {
		cerr << "exception caught: " << err.why() << endl;
	}
}

// Requires: num[] is sorted, high, low in range of num[]
// Ensures: (result.keyFound == 0)
//		or ((result.keyfound == 1) and (num[result.index] = key))
keyIndex binSearch(int key, int num[], int low, int high)
{
	keyIndex result(0,0);
	if (low > high) {
		return keyIndex(0,0);			// not found
	} else if (low == high) {
		if (key == num[high]) {
			return keyIndex(1,high);	// found at position high (== low)
		} else {
			return keyIndex(0,0);		// not found
		}
	} else { // high > low
		int mid = (high+low)/2;			// => mid < high
		if (key <= num[mid]) {
			return binSearch(key, num, low, mid);
		} else {
			return binSearch(key, num, mid+1, high);
		}
	}
}

// Iterative version obtained by transforming tail-recursion ...

// Requires: num[] is sorted, high, low in range of num[]
// Ensures: (result.keyFound == 0)
//		or ((result.keyfound == 1) and (num[result.index] = key))
keyIndex ibinSearch(int key, int num[], int low, int high)
{
	keyIndex result(0,0);
	
	while (low <= high) {
		if (low == high) {
			if (key == num[high]) {
				return keyIndex(1,high);	// found at position high (== low)
			} else {
				return keyIndex(0,0);		// not found
			}
		} else { // high > low
			int mid = (high+low)/2;			// => mid < high
			if (key <= num[mid]) {
				// return binSearch(key, num, low, mid);
				high = mid;
			} else {
				// return binSearch(key, num, mid+1, high);
				low = mid+1;
			}
		}
	}
	return keyIndex(0,0);			// not found
}
