
// FILE: mergeSort.cpp

#include <iostream.h>
#include <exception.h>
#include "printArray.h"
#include "mergeSort.h"

static void doTest(int a[], int len);

static void merge(int a1[], int l1, int a2[], int l2, int b[]);

void mergeSort2(int a[], int len);

static void ms2a(int a[], int b[], int len);
static void ms2b(int a[], int b[], int len);
void runTest(void (*msort) (int[], int));

static const int testSize = 100000;

void testSort1() { runTest(mergeSort); }
void testSort2() { runTest(mergeSort2); }

void runTest(void (*msort) (int[], int))
{
	int a[testSize];
	int i;
	for (i=0;i<testSize;i++) {
		a[i] = testSize - i;
	}
	msort(a, testSize);
}

void testSort()
{
	int a0[] = { 19, 3, 5, 22, 7, 7, 1 };
	doTest(a0, 7);
	int a1[] = {2, 2, 2};
	doTest(a1, 3);
	doTest(a1, 0);
}

void doTest(int a[], int len)
{
	try {
		cout << "mergeSort(";
		printArray(cout,a,len);
		cout << ") = ";
		mergeSort(a, len);
		printArray(cout,a,len);
		cout << endl;
		
		cout << "mergeSort2(";
		printArray(cout,a,len);
		cout << ") = ";
		mergeSort2(a, len);
		printArray(cout,a,len);
		cout << endl;		
	} catch (xmsg &err) {
		cerr << "exception caught: " << err.why() << endl;
	}
}

// Requires: a is an array of ints, length len
// Ensures: a will be sorted
void mergeSort(int a[], int len)
{
	// This version is nice and simple, but copies a lot
	if (len <= 1) {
		return;
	}

	int *a1 = a;
	int l1 = len/2;
	mergeSort(a1, l1);

	int *a2 = a + l1;
	int l2 = len - l1;
	mergeSort(a2, l2);

	int *b = new int[len];
	merge(a1, l1, a2, l2, b);
	int i;
	for (i=0; i<len; i++) {
		a[i] = b[i];
	}
	delete [] b;
}

// Requires: a1 and a2 are sorted arrays of length l1 and l2 resp
// Ensures: b will contain sorted merge of a1 and a2
void merge(int a1[], int l1, int a2[], int l2, int b[])
{
	int i1 = 0;
	int i2 = 0;
	int len = l1 + l2;
	int i;
	for (i=0; i<len; i++) {
		// Invariant: (i == i1 + i2) && (len = l1 + l2)
		/*
		if (i1 < l1) {
			if (i2 >= l2) {
				b[i] = a1[i1++];
			} else if (a1[i1] <= a2[i2]) {
				b[i] = a1[i1++];
			} else {
				b[i] = a2[i2++];
			}
		} else {
			b[i] = a2[i2++];
		}
		*/
		if ((i1 < l1) && ((i2 >= l2) || (a1[i1] <= a2[i2]))) {
			b[i] = a1[i1++];
		} else {
			b[i] = a2[i2++];
		}
	}
}

// This version does less copying, but is more complex
void mergeSort2(int a[], int len)
{
	int *b = new int[len];
	ms2a(a, b, len);
	delete [] b;
}

// Ensures: a will be sorted
void ms2a(int a[], int b[], int len)
{
	if (len <= 1) {
		return;
	}

	int *a1 = a;
	int *b1 = b;
	int l1 = len/2;
	ms2b(a1, b1, l1);

	int *a2 = a + l1;
	int *b2 = b + l1;
	int l2 = len - l1;
	ms2b(a2, b2, l2);

	merge(b1, l1, b2, l2, a);
}

// Ensures: b will be a sorted
void ms2b(int a[], int b[], int len)
{
	if (len <= 1) {
		if (len == 1) {
			b[0] = a[0];
		}
		return;
	}

	int *a1 = a;
	int *b1 = b;
	int l1 = len/2;
	ms2a(a1, b1, l1);

	int *a2 = a + l1;
	int *b2 = b + l1;
	int l2 = len - l1;
	ms2a(a2, b2, l2);

	merge(a1, l1, a2, l2, b);
}
