
/*
	File: Debug.h
	(c) Oscar Nierstrasz 30.3.96
	
	Debug printing package.
	Print to the global variable "debug" as you would
	to cout or cerr.  (Actually prints to cerr)
	#define DEBUG to turn on debug printing

	WARNING: for non-string/endl messages, a template function
	is used -- with some compilers, only one instance can be
	generated (even if it is declared inline), so you cannot have
	DEBUG on in one file and off in another :-(
	The best solution is to enclose every debug stmt inside
	an #ifdef DEBUG .. #endif
*/

#ifndef DEBUG_H
#define DEBUG_H

#include <iostream.h>

class Debug
{
  public:
	Debug (void) : needLeader(1) { };		// initially need leader
	void printLeader(void);					// print leader if necessary
	Debug& endl (ostream&(*) (ostream&));	// forward endl to cerr
  private:
	~Debug(void)  { };						// never destruct
	int needLeader;							// need to print leader
};

extern Debug debug;							// global instance

#ifdef DEBUG

// Forward endl to debug stream:
inline Debug&
operator<<(Debug& db, ostream&(*op) (ostream&))
{
	return db.endl(op);
}

// String msgs (defined to avoid template instantiation problem)
inline Debug&
operator<<(Debug& db, const char * msg)
{
	db.printLeader();	// only if needed
	cerr << msg;
	return db;
}

// For all other msgs, forward to cerr
template <class Message>
inline Debug&
operator<<(Debug& db, const Message msg)
{
	db.printLeader();	// only if needed
	cerr << msg;
	return db;
}

#else

// If DEBUG is off, nothing is executed.
// "return" db to keep type checker happy.
inline Debug&
operator<<(Debug& db, ostream&(*op) (ostream&))
{	
	return db;
}

inline Debug&
operator<<(Debug& db, const char * msg)
{
	return db;
}

template <class Message>
inline Debug&
operator<<(Debug& db, const Message msg)
{
	return db;
}

#endif // DEBUG

#endif // DEBUG_H

