
/*
	File: Matrix.h
	(c) Oscar Nierstrasz 31.3.96

	A generic Matrix class.
*/

#ifndef MATRIX_H
#define MATRIX_H

#include <exception.h>
#include <iostream.h>

// #define DEBUG
#include "Debug.h"
#include "Array.h"

template <class Item>
class Matrix
{
public:
	Matrix (int rows=0, int cols=0);
	Matrix (int rows, int cols, const Item& initVal);
	
	// the default copy constructor and operator= fine!

	~Matrix (void) { } ; // nothing to do!

	Array<Item>& operator[] (const int n) throw(xmsg) const { return matrixRep[n]; }
	
	int rows (void) const { return matrixRep.size(); }
	int cols (void) const { return matrixRep[0].size(); }
	void resize (int rows, int cols) throw(xmsg);

protected:
	Array<Array<Item> > matrixRep;
};

// default constructor
template <class Item>
Matrix<Item>::Matrix(int rows, int cols)
{
	debug << "Matrix::Matrix -- Made new matrix. rows = " << rows;
	debug << ", cols = " << cols << endl;
	this->resize(rows, cols);
}

// initializing constructor
template <class Item>
Matrix<Item>::Matrix(int rows, int cols, const Item& initVal)
{
	debug << "Matrix::Matrix -- Made new matrix. rows = " << rows;
	debug << ", cols = " << cols << endl;
	this->resize(rows, cols);
	for (int r=0; r<rows; r++) {
		for (int c=0; c<cols; c++) {
			matrixRep[r][c] = initVal;
		}
	}
}

template <class Item>
void
Matrix<Item>::resize(int rows, int cols)
{
	debug << "Matrix::resize -- rows = " << rows;
	debug << ", cols = " << cols << endl;
	matrixRep.resize(rows);
	for (int r=0;r<rows;r++)  {
		matrixRep[r].resize(cols);
	}
}

#endif // MATRIX_H
