
/*
	File: BoardGame.cpp
	Author: Oscar Nierstrasz 18.3.96

	Generic BoardGame operations
*/


#include "BoardGame.h"

// Constructor:
BoardGame::BoardGame (int rs, int cs) throw(xmsg) :
	rows(rs), cols(cs),
	_winner(nobody),
	_turn(X),
	squaresLeft(rs*cs)
{
	debug("calling BoardGame constructor");

	square = new Player* [rows];
	if (square == 0)
		throw(xmsg("Can't allocate board"));

	for (int r=0; r<rows; r++)  {
		square[r] = new Player [cols];
		if (square[r] == 0)
			throw(xmsg("Can't allocate board"));
		for (int c=0; c<cols; c++)
			square[r][c] = nobody;
	}
}

// Destructor:
BoardGame::~BoardGame (void)
{
	debug("calling BoardGame destructor");

	for (int r=0; r<rows; r++)
		delete [] square[r];
	delete [] square;
}

// Game isn't over until there is a winner:
int
BoardGame::notover (void)
{
	// cerr << "There are " << squaresLeft << " squares left" << endl;
	// cerr << "winner is " << winners[_winner] << endl;

	return (squaresLeft > 0) && (_winner == nobody);
}

// String representation of winner:
// Result string may not be modified by clients!
const char *
BoardGame::winner (void)
{
	return winners[_winner];
}

// Char representation of current player:
char
BoardGame::turn (void)
{
	return player[_turn];
}

// Char and string names of players.
// Initialization of constant static members:
const char BoardGame::player [3] =  { ' ', 'X', 'O' };
const char * BoardGame::winners [3] =  { "Nobody", "X", "O" };

/*
	Current player makes a move by marking a square labelled aA-zZ.
	An exception is raised if the square is out of range
	or is already marked.
*/
void
BoardGame::play (String& move) throw(xmsg)
{
	if (!notover()) {
		throw(xmsg("This game is already over!"));
		return;
	}

	if (move.strlen() != 2) {
		throw(xmsg("Improper response: please give coordinates [a-z][A-Z]"));
	}

	// Check if move is in range, and convert to index into square[][]
	int row = move[0] - 'a';
	int col = move[1] - 'A';
	if (!inRange(row, col)) {
		throw(xmsg("Row out of range"));
	}

	makeMove (row, col);
	checkWinner (row, col);					// maintain invariants
}

int
BoardGame::inRange (int row, int col)
{
	return (0<=row) && (row<rows) && (0<=col) && (col<cols);
}

/*
	The default assumes you can mark any empty square.
	Override this to implement a different logic for valid moves.
*/
void
BoardGame::makeMove (int row, int col) throw(xmsg)
{
	if (square[row][col] == nobody)  {		// not already marked
		square[row][col] = _turn;			// mark the square
		squaresLeft--;
		_turn = (_turn == X) ? O : X;		// switch current player
	} else {
		throw(xmsg("Square already occupied"));
	}
}

// Pretty print the current state of the game:
void
BoardGame::print (void)
{
	static const char * tabs = "\t\t"; // initial tabs
	cout << endl;
	// print the column labels
	cout << tabs << "    ";
	int col;
	for (col=0; col<cols; col++)
		cout << char('A' + col) << "   "; // each column has a label A-Z
	cout << endl;
	// print the game row by row
	for (int row=0; row<rows; row++) {
		// print a row separator
		cout << tabs << "  +";
		for (col=0; col<cols; col++)
			cout << "---+";
		cout << endl;
		// print the squares
		cout << tabs << char('a' + row) << " |"; // each row has a label a-z
		for (col=0; col<cols; col++)
			cout << " " << player[square[row][col]] << " |";
		cout << endl;
	}
	// print a final row separator
	cout << tabs << "  +";
	for (col=0; col<cols; col++)
		cout << "---+";
	cout << endl << endl;
}

/*
	Helper function for BoardGame::play()
	Check for three in a row in all directions.
*/
void
BoardGame::checkWinner (int row, int col)
{
	// pure virtual -- just need a definition
}

// Convenience.
// We could also have defined this as a friend function
// and avoided the seperate print() member.
ostream&
operator<< (ostream& os, BoardGame& game)
{
	game.print();
	return os;
}
