
/*
	File: Gomoku.cpp
	Author: Oscar Nierstrasz 29.2.96
	
	Go-moku implementation
	NB: Internally all squares are numbered 0-8; externally 1-9.
*/


#include "Gomoku.h"

// only needed locally, so declared as static
// don't want to conflict with any global right, left ...
static void right (int&, int&);
static void left (int&, int&);
static void up (int&, int&);
static void down (int&, int&);
static void northeast (int&, int&);
static void southwest (int&, int&);
static void northwest (int&, int&);
static void southeast (int&, int&);

// Constructor:
Gomoku::Gomoku (int r, int c, int ws) throw(xmsg) :
	BoardGame(r,c),
	winningScore(ws)
{
	debug("calling Gomoku constructor");
}

// Destructor:
Gomoku::~Gomoku (void)
{
	debug("calling Gomoku destructor");
}

/*
	Helper function for Gomoku::play()
	Check for three in a row in all directions.
*/
void
Gomoku::checkWinner (int row, int col)
{
	checkScore(row, col, right, left);
	checkScore(row, col, up, down);
	checkScore(row, col, northeast, southwest);
	checkScore(row, col, northwest, southeast);
}

void
Gomoku::checkScore (int row, int col,
	void (* thisMove) (int&, int&),
	void (* thatMove) (int&, int&))
{
	int score = 1;
	int r=row, c=col;
	// check in this direction
	thisMove(r,c);				// same as: (*thisMove)(r,c);
	while (inRange(r,c) && square[r][c] == square[row][col])  {
		// cerr << "Checking (" << row << "," << col << ")" << endl;
		
		score++;
		thisMove(r,c);
	}
	// cerr << "Stopped at (" << row << "," << col << ")" << endl;

	// and now in that direction
	r = row;
	c = col;
	thatMove(r,c);
	while (inRange(r,c) && square[r][c] == square[row][col])  {
		// cerr << "Checking (" << row << "," << col << ")" << endl;
		score++;
		thatMove(r,c);
	}
	// cerr << "Stopped at (" << row << "," << col << ")" << endl;
	// cerr << "Score = " << score << endl;
	
	if (score >= winningScore)  {
		_winner = square[row][col];
	}
}

void right (int& row, int& col) { col++; }
void left (int& row, int& col) { col--; }

void up (int& row, int& col) { row++; }
void down (int& row, int& col) { row--; }

void northeast (int& row, int& col) { row++; col++; }
void southwest (int& row, int& col) { row--; col--; }

void northwest (int& row, int& col) { row++; col--; }
void southeast (int& row, int& col) { row--; col++; }



