
// dstore.h
// Author: Oscar Nierstrasz 30.11.95
// A dynamic store modelled after Perl arrays

#ifndef DSTORE_H
#define DSTORE_H

#include <exception.h>
#include <iostream.h>

// typedef char* Item; // Redefine as necessary (alternative to templates)

static const int defSize = 32;

template <class Item>
class DStore
{
public:
	DStore(int initSize=defSize) { become(initSize); };
	~DStore(void);
	// copy constructor:
	DStore(const DStore& copy) { become(defSize, copy.curSize, copy._store); };

	DStore(const elts, const Item * initStore) { become(defSize, elts, initStore); };

	int size(void) const { return curSize; }
	int empty(void) const { return curSize == 0; };

	Item& operator[](const int n) const;

	DStore& operator=(const DStore&);

	void push(const Item& item); // throw();
	Item& top(void) const; // throw(xmsg);
	void pop(void); // throw(xmsg);
	void zero(void) { curSize = 0; };

	void stats(void);

	// Will also need shift, unshift ...
	
	// needed for DString ...
	const Item * array(void) { return _store; };

protected:
	int curSize;
	int maxSize;
	Item * _store;

	// Helper members:

	void grow(void);
	Item * makeDStore(int);
	void become(int initSize, int n=0, const Item* values = 0);
};

template <class Item>
DStore<Item>::~DStore(void)
{
	delete [] _store;
}

template <class Item> Item&
DStore<Item>::operator[](const int n) const
{
	if (n >= curSize) {
		throw(xmsg("index too large"));
	}
	return this->_store[n];
}

template <class Item> DStore<Item>&
DStore<Item>::operator=(const DStore<Item>& val)
{
	if (this != &val) {
		delete [] _store;
		become(val.maxSize, val.curSize, val._store);
	}
	return *this; // MPW caught this
}

template <class Item> void
DStore<Item>::push(const Item& item)
{
	if (curSize == maxSize) {
		this->grow();
	}
	_store[curSize++] = item;
}

template <class Item> Item&
DStore<Item>::top(void) const
{
	if (this->empty()) {
		throw(xmsg("Empty stack has no top!"));
	}
	return _store[curSize-1];
}

template <class Item> void
DStore<Item>::pop(void)
{
	if (this->empty()) {
		throw(xmsg("Can't pop an empty stack!"));
	}
	curSize--;
	//
	// if too small, then shrink ...
	//
}

template <class Item> void
DStore<Item>::grow(void)
{
	Item * oldDStore;
	oldDStore = _store;
	_store = makeDStore(maxSize*2);
	maxSize = maxSize*2;
	for (int i=0; i<curSize; i++) {
		_store[i] = oldDStore[i];
	}
	delete [] oldDStore;
	return;
}

template <class Item> Item *
DStore<Item>::makeDStore(int newSize)
{
	Item * newDStore = new Item[newSize];
	if (newDStore == 0) {
		throw(xmsg("DStore<Item>::makeDStore: Couldn't allocate enough space"));
	}
	return newDStore;
}

template <class Item> void
DStore<Item>::become(int initSize, int n, const Item* values)
{
	while (initSize < n) {
		initSize = 2*initSize;
	}
	if (n > 0 && values == 0) {
		throw(xmsg("become: n>0 but values=0!"));
	}
	// NB: if already initialized, caller must first delete [] _store!

	maxSize = initSize;
	curSize = n;
	_store = makeDStore(maxSize);
	for (int i=0; i<n; i++) {
		_store[i] = values[i];
	}
}

template <class Item> void
DStore<Item>::stats(void)
{
	cout << "maxSize = " << maxSize
			<< "; curSize = " << curSize
			<< endl;;
}

#endif

