
// File: stlTest.cpp
// Author: Oscar Nierstrasz 4.5.96
// Line reverser implemented using STL

#include <iostream.h>

#include <pair.h>
#include <assert.h>

#include <stack.h>
#include <vector.h>
#include <algo.h> // for sort()

#include <multimap.h>

#define DEBUG

#include "String.h"

void pairTest(void);

void copyTest(void);
void rev1(void);
void rev2(void);
void rev3(void);
void strSort(void);
void revSort(void);
void acronym(void);

int main(void)
{
	// copyTest();
	// rev1();
	// revSort();
	// pairTest();
	acronym();
	return 0;
}

void acronym(void)
{
	String input, copy;
	multimap<String, String> dict;

	do {
		cout << "Enter string: " << flush;
		cin >> input;
		copy = input;

		sort(copy.begin(), copy.end());
		// NB: String augmented with begin() & end()

		cout << "Sorted = `" << copy << "'" << endl;
	} while (input.strlen() != 0);
	cout << "Bye!" << endl;
}

template <class Pair>
void printPair(char* name, Pair p)
{
	cout << name << " = pair(" << p.first << "," << p.second << ")" << endl;
}

void pairTest(void)
{
	pair<int,int> rect(3,5), rect2(2,6);
	printPair("rect", rect);
	assert(rect2<rect);

	typedef pair<int, String> intStrPair;
	intStrPair red(1, "red"), blue(2, "blue"), colour;

	printPair("red", red);
	colour = blue;
	printPair("colour", colour);

	assert(colour == blue);
	// NB: must define operator== for String, otherwise:
	// Error   : illegal operand
	// pair.h line 31   return x.first == y.first && x.second == y.second; 
}

void findValue(vector<int> vec, int val);

void copyTest(void)
{
	vector<int> a(10), b(5);
	for (int i=0; i<10; i++) {
		a[i] = i*i;
	}
	copy(a.begin()+5, a.end(), b.begin());
	for (int j=0; j<5; j++)  {
		cout << "b[" << j << "] = " << b[j] << endl;
	}
	findValue(a, 25);				// found value 25
	findValue(b, 99);				// value 99 not found
}

void findValue(vector<int> vec, int val)
{
	vector<int>::iterator it;
	it = find(vec.begin(), vec.end(), val);
	if (it != vec.end()) {
		cout << "found value " << val << endl;
	} else {
		cout << "value " << val << " not found" << endl;
	}
}

void rev1(void)
{
	typedef stack<vector<String> > IOStack;
	IOStack ioStack;
	String buf;

	while (cin >> buf) {
		ioStack.push(buf);
	}
	while (ioStack.size() != 0) {
		cout << ioStack.top() << endl;
		ioStack.pop();
	}
}

void rev2(void)
{
	typedef vector<String> IOVector;
	typedef istream_iterator<String, ptrdiff_t> String_in;

	IOVector ioVec;

	copy (String_in(cin), String_in(), back_inserter(ioVec));
	reverse(ioVec.begin(), ioVec.end());
	copy (ioVec.begin(), ioVec.end(),
		ostream_iterator<String>(cout, "\n"));
}

void rev3(void)
{
	typedef vector<String> IOVector;
	typedef istream_iterator<String, ptrdiff_t> String_in;

	IOVector ioVec;

	copy(
		String_in(cin),				// input stream iterator
		String_in(),				// "past the end" iterator for input
		back_inserter(ioVec));		// append instead of overwrite

	copy(ioVec.rbegin(),			// reverse direction: begin at end
		ioVec.rend(),				// end at beginning
		ostream_iterator<String>(cout, "\n"));	// output stream iterator
}

void strSort(void)
{
	typedef vector<String> IOVector;
	typedef istream_iterator<String, ptrdiff_t> String_in;

	IOVector ioVec;

	copy(String_in(cin), String_in(), back_inserter(ioVec));
	sort(ioVec.begin(), ioVec.end()); // NB: assumes < defined for String
	copy(ioVec.begin(), ioVec.end(),
		ostream_iterator<String>(cout, "\n"));
}

// Function object to compare Strings in reverse direction
class revStrCmp
{
public:
	bool operator() (const String& s1, const String& s2) { return s2 < s1; }
};

bool revStrCmpFn (const String& s1, const String& s2);

bool revStrCmpFn (const String& s1, const String& s2) { return s2 < s1; }

void revSort(void)
{
	typedef vector<String> IOVector;
	typedef istream_iterator<String, ptrdiff_t> String_in;

	IOVector ioVec;

	copy(String_in(cin), String_in(), back_inserter(ioVec));
	sort(ioVec.begin(), ioVec.end(),
		revStrCmp());				// pass an instance of revStrCmp
		// revStrCmpFn);			// pass a function pointer
	copy(ioVec.begin(), ioVec.end(),
		ostream_iterator<String>(cout, "\n"));
}
