
// FILE: postfix.cpp

#include <iostream.h>
#include <exception.h>

#include "postfix.h"
#include "stack.h"

static void applyOp(int (*stackOp) (int, int), Stack &intStack);

static int plus(int a1, int a2);
static int minus(int a1, int a2);
static int times(int a1, int a2);
static int divide(int a1, int a2);
static int mod(int a1, int a2);

void postfix(void)
{
	Stack intStack;

	char c = ' ';
	cout << "Enter postfix expressions (\".\" to stop!)" << endl;
	while (c != '.') {
		try {
			int arg1, arg2;
			cin >> c;
			if (('0'<=c) && (c<='9')) {
				intStack.push(c - '0');
				cout << "Pushed " << c << endl;
			} else {
				switch (c) {
				case '+':
					arg1 = intStack.top();
					intStack.pop();
					arg2 = intStack.top();
					intStack.pop();
					intStack.push(arg1 + arg2);
					cout << arg1 << " + " << arg2 << " = " << (arg1+arg2) << endl;
					break;
				case '-':
					applyOp(minus, intStack);
					break;
				case '*':
					applyOp(times, intStack);
					break;
				case '/':
					applyOp(divide, intStack);
					break;
				case '%':
					applyOp(mod, intStack);
					break;
				// For debugging ...
				case '=':
					cout << intStack << endl;
					// intStack.print(cout); cout << endl;
					break;
				default:
					cerr << "Invalid char " << c << " ignored" << endl;
					break;
				}
			}
		} catch (xmsg &err) {
			cout << "Exception: " << err.why() << endl;
		}
	}
}

void applyOp(int (*stackOp) (int, int), Stack& intStack)
{
	int arg1 = intStack.top();
	intStack.pop();
	int arg2 = intStack.top();
	intStack.pop();
	int result = stackOp(arg1, arg2);
	intStack.push(result);
	cout << "Popped " << arg1 << " and " << arg2
		<< "; pushed " << result << endl;
}

int plus(int a1, int a2) { return a1 + a2; }
int minus(int a1, int a2) { return a1 - a2; }
int times(int a1, int a2) { return a1 * a2; }
int divide(int a1, int a2) { return a1 / a2; }
int mod(int a1, int a2) { return a1 % a2; }
