
/*
	File: TicTacToe.cpp
	Author: Oscar Nierstrasz 29.2.96
	
	Tic Tac Toe implementation
	NB: Internally all squares are numbered 0-8; externally 1-9.
*/

#include <iostream.h>
#include "TicTacToe.h"

// Constructor:
TicTacToe::TicTacToe(void) :
	_winner(nobody),
	_turn(X),
	squaresLeft(9)
{
	for (int i=0; i<9; i++) square[i] = nobody;
}

// Game isn't over until there is a winner:
int
TicTacToe::notover(void)
{
	return (squaresLeft > 0) && (_winner == nobody);
}

// String representation of winner:
// Result string may not be modified by clients!
const char *
TicTacToe::winner (void)
{
	return winners[_winner];
}

// Char representation of current player:
char
TicTacToe::turn(void)
{
	return player[_turn];
}

// Char and string names of players.
// Initialization of constant static members:
const char TicTacToe::player [3] =  { ' ', 'X', 'O' };
const char * TicTacToe::winners [3] =  { "Nobody", "X", "O" };

// Alternatively, they could be static global variables:
// static const char player [] =  { ' ', 'X', 'O' };
// static const char * winners [] =  { "Nobody", "X", "O" };

/*
	Current player makes a move by marking a square from 1-9.
	An exception is raised if the square is out of range
	or is already marked.
*/
void
TicTacToe::play(int move) throw(xmsg)
{
	if (!notover()) {
		throw(xmsg("This game is already over!"));
		return;
	}

	if ((move<1) || (move>9)) {
		throw(xmsg("Move out of range 1-9"));
		return;
	}
	move--; // OK, so decrement (index square from 0-8)
	
	if (square[move] == nobody)  {		// not already marked
		square[move] = _turn;		// mark the square
		squaresLeft--;

		_turn = (_turn == X)? O : X;	// switch current player
		checkWinner();			// maintain invariants
	} else {
		throw(xmsg("Square already occupied"));
	}
}

// Pretty print the current state of the game:
void
TicTacToe::print(void)
{
	cout << endl;
	for (int row=0; row<3; row++) {	// print the game row by row
		int first = 3*row;
		cout << '\t'
			<< showSquare(first) << '|'
			<< showSquare(first+1) << '|'
			<< showSquare(first+2) << endl;
		if (row < 2) cout << "\t-----" << endl;
	}
	cout << endl;
}

/*
	Helper function for TicTacToe::print()
	Return ascii char for squares 0-8
	Returns 'X' or 'O' if occupied; otherwise square number as ascii char
*/
char
TicTacToe::showSquare(int m)
{
	Player state = square[m];
	return (state == nobody)?('1'+m):player[state];
}

/*
	Helper function for TicTacToe::play()
	Check for three in a row in all directions.
*/
void
TicTacToe::checkWinner(void)
{
	// check the rows:
	for (int row=0; row<3; row++) {
		int first = 3*row;
		if (matchThree(first, first+1, first+2))  {
			return;
		}
	}
	// check the columns:
	for (int col=0; col<3; col++) {
		if (matchThree(col, col+3, col+6)) {
			return;
		}
	}
	// check the diagonals:
	if (matchThree(0, 4, 8)) return;
	if (matchThree(2, 4, 6)) return;
}

/*
	Helper function for TicTacToe::checkWinner()
	Set winner if some player has three in a row
	NB: implies game is over.
*/
int
TicTacToe::matchThree(int first, int second, int third)
{
	if (square[first] == nobody) {
		return 0;
	}
	
	if ((square[first] == square[second])
		&& (square[second] == square[third])) {
		_winner = square[first];
		return 1;
	} else {
		return 0;
	}
}

/*
	Friend function to support stream-style printing.
	Cannot be a member function of TicTacToe since target is an ostream.
	TicTacToe::showSquare() is private, so it must be a friend.
*/
ostream&
operator<<(ostream& os, TicTacToe& game)
{
	os << endl;
	for (int row=0; row<3; row++) {	// print the game row by row
		int first = 3*row;

		os << '\t'
			<< game.showSquare(first) << '|'
			<< game.showSquare(first+1) << '|'
			<< game.showSquare(first+2) << endl;
		if (row < 2) os << "\t-----" << endl;
	}
	os << endl;
	return os;
}

